/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.controllers;

import java.awt.Color;
import java.io.IOException;
import fr.amap.lidar.amapvox.visualization.views.VisualizationScene;
import fr.amap.lidar.amapvox.visualization.opengl.util.ColorBars;
import fr.amap.lidar.amapvox.visualization.opengl.util.Skyboxes;
import fr.amap.lidar.amapvox.visualization.util.ProjectProperties;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class RenderingController {

    private VisualizationScene scene;

    public RenderingController() {
        this.scene = null;
    }

    public final void setScene(final VisualizationScene scene) {
        this.scene = scene;
    }

    public final void changeBackground(final String background)
            throws IOException {
        if (scene != null) {
            scene.setBackground(Skyboxes.getSky(background));
            ProjectProperties.setProperty("background", background);
        }
    }

    public final void changeColorBar(final String name) {
        final Color[] colorBar = ColorBars.getColorBar(name);
        scene.getVoxelSpaceComponent().setColorBar(colorBar);
        scene.setSomethingsModified();
        ProjectProperties.setProperty("colorBar", name);
    }

    public final void changeDrawFast(final boolean enabled) {
        scene.getVoxelSpaceComponent().setDrawFastEnabled(enabled);
        ProjectProperties.setProperty("drawFastEnabled", Boolean.toString(enabled));
    }
    
    public final void changeVoxelSize(final int size) {
        final float normalizedSize = (float) size / 100;
        scene.getVoxelSpaceComponent().setDrawingMode(normalizedSize);
        ProjectProperties.setProperty("voxelSize", Integer.toString(size));
    }
    
    public final void changeEnableReflection(final boolean enabled) {
        if (scene != null) {
            scene.getVoxelSpaceComponent().setReflectionEnabled(enabled);
            ProjectProperties.setProperty("reflectionsEnabled", Boolean.toString(enabled));
        }
    }

}
