/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.visualization.opengl.entities.extended;

import org.joml.Matrix4f;
import org.joml.Vector3f;
import fr.amap.lidar.amapvox.visualization.opengl.entities.base.Component;
import fr.amap.lidar.amapvox.visualization.opengl.resources.materials.Material;
import fr.amap.lidar.amapvox.visualization.opengl.resources.materials.StandardMaterial;
import fr.amap.lidar.amapvox.visualization.opengl.resources.meshes.Mesh;
import fr.amap.lidar.amapvox.visualization.opengl.resources.programs.Program;
import fr.amap.lidar.amapvox.visualization.opengl.resources.textures.TextureCubemap;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public abstract class Renderer extends Component {
    
    /**
     * The current used material.
     */
    private Material material;
    
    /**
     * The mesh to draw.
     */
    private Mesh mesh;

    public Renderer() {
        this.material = null;
        this.mesh = null;
    }
    
    /**
     * Change the current material.
     * @param material 
     */
    public final void setMaterial(final Material material) {
        this.material = material;
    }

    /**
     * Change the current mesh.
     * @param mesh 
     */
    public final void setMesh(final Mesh mesh) {
        this.mesh = mesh;
    }

    /**
     * Get the current material.
     * @return 
     */
    public final Material getMaterial() {
        return material;
    }

    /**
     * Get the current mesh.
     * @return 
     */
    public final Mesh getMesh() {
        return mesh;
    }
    
    /**
     * Generate a program of this renderer with its material and mesh.
     * @return 
     */
    public final Program getProgram() {
        if (material == null || mesh == null) {
            throw(new NullPointerException("The current material or mesh is empty."));
        }
        
        final Program program = material.generateProgram();
        program.setMesh(mesh);
        return program;
    }

    @Override
    protected final void receiveMessage(final String name, final Object object) {
        if (null != name) switch (name) {
            case "viewMatrix":
                material.setView((Matrix4f) object);
                break;
            case "projectionMatrix":
                material.setProjection((Matrix4f) object);
                break;
            case "viewPosition":
                material.setViewPosition((Vector3f) object);
                break;
            case "background":
                if (material instanceof StandardMaterial) {
                    ((StandardMaterial) material).setCubemap((TextureCubemap) object);
                } break;
        }
    }
    
}
