/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.visualization.opengl.resources.programs;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL3bc;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import fr.amap.lidar.amapvox.visualization.opengl.resources.materials.StandardMaterial;
import fr.amap.lidar.amapvox.visualization.opengl.resources.shaders.Shader;
import fr.amap.lidar.amapvox.visualization.opengl.resources.textures.Texture2D;
import fr.amap.lidar.amapvox.visualization.opengl.resources.textures.TextureCubemap;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class StandardProgram extends Program {

    Texture2D albedo = null;
    TextureCubemap cubemap = null;
    boolean reflectionEnabled = true;
    float[] mainColorValues = new float[materials.size() * 4];

    private int albedoLocation;
    private int cubemapLocation;
    private IntBuffer vbo;

    public StandardProgram() {
        super("StandardProgram");
        this.vbo = Buffers.newDirectIntBuffer(1);
        shader = new Shader("StandardShader");
        shader.setVertexSources(
                "/shaders/preprocessing/include/vertex.glsl",
                "/shaders/preprocessing/standard/vertex.glsl"
        );
        shader.setFragmentSources(
                "/shaders/preprocessing/include/fragment.glsl",
                "/shaders/preprocessing/standard/fragment.glsl"
        );
    }

    public final void setReflectionEnabled(boolean reflectionEnabled) {
        this.reflectionEnabled = reflectionEnabled;
    }

    @Override
    protected void onLoad(GL3bc gl) {
        super.onLoad(gl);

        gl.glGenBuffers(vbo.limit(), vbo);

        gl.glBindBuffer(GL3bc.GL_ARRAY_BUFFER, vbo.get(0));
        gl.glEnableVertexAttribArray(5);
        gl.glVertexAttribPointer(5, 4, GL3bc.GL_FLOAT, false, 0, 0);
        gl.glVertexAttribDivisor(5, 1);

        albedoLocation = gl.glGetUniformLocation(shader.getProgram(), "albedo");
        gl.glUniform1i(albedoLocation, 0);
        cubemapLocation = gl.glGetUniformLocation(shader.getProgram(), "cubemap");
        gl.glUniform1i(cubemapLocation, 1);
    }

    @Override
    public synchronized void updateProperties() {
        super.updateProperties();

        /* Get materials values */
        mainColorValues = new float[materials.size() * 4];

        for (int i = 0; i < materials.size(); i++) {
            final StandardMaterial material = (StandardMaterial) materials.get(i);

            if (material != null) {
                if (material.isHiden()) {
                    mainColorValues[i * 4 + 0] = 1.0f;
                    mainColorValues[i * 4 + 1] = 1.0f;
                    mainColorValues[i * 4 + 2] = 1.0f;
                    mainColorValues[i * 4 + 3] = 0.0f;
                } else {
                    mainColorValues[i * 4 + 0] = (float) 
                            material.getMainColor().getRed() / 255;
                    mainColorValues[i * 4 + 1] = (float) 
                            material.getMainColor().getGreen() / 255;
                    mainColorValues[i * 4 + 2] = (float) 
                            material.getMainColor().getBlue() / 255;
                    mainColorValues[i * 4 + 3] = (float) 
                            material.getMainColor().getAlpha() / 255;
                }

                cubemap = material.getCubemap();
                albedo = material.getAlbedo();
            }
        }
    }

    @Override
    protected void onBind(GL3bc gl) {
        super.onBind(gl);

        /* Enable model transformation matrix */
        gl.glBindBuffer(GL3bc.GL_ARRAY_BUFFER, vbo.get(0));

        final FloatBuffer modelBuf
                = Buffers.newDirectFloatBuffer(mainColorValues);
        gl.glBufferData(GL3bc.GL_ARRAY_BUFFER, Float.BYTES * modelBuf.limit(),
                modelBuf, GL3bc.GL_STATIC_DRAW);

        /* Send albedo texture */
        gl.glActiveTexture(GL3bc.GL_TEXTURE0);

        if (albedo != null) {
            albedo.bind(gl);
        } else {
            gl.glBindTexture(GL3bc.GL_TEXTURE_2D, 0);
        }

        /* Send cubemap texture */
        gl.glActiveTexture(GL3bc.GL_TEXTURE1);

        if (cubemap != null && reflectionEnabled) {
            cubemap.bind(gl);
        } else {
            gl.glBindTexture(GL3bc.GL_TEXTURE_CUBE_MAP, 0);
        }
    }

}
