package fr.amap.lidar.amapvox.visualization.opengl.resources.textures;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL3bc;
import java.nio.IntBuffer;
import fr.amap.lidar.amapvox.visualization.opengl.resources.Resource;

public abstract class Texture extends Resource {
        
    protected final IntBuffer tbo;
    protected final int width;
    protected final int height;
    protected TextureFormat format;
    protected boolean generateMipmaps;
    
    public Texture(final String identifier, final int width, final int height,
            final TextureFormat format, final boolean generateMipmaps) {
        super(identifier);
        this.tbo = Buffers.newDirectIntBuffer(1);
        this.width = width;
        this.height = height;
        this.format = format;
        this.generateMipmaps = generateMipmaps;
    }

    public final int getIndex() {
        return tbo.get(0);
    }
    
    protected final int getInternalFormat() {
        if (format != null) switch(format) {
            case RGB: return GL3bc.GL_RGB;
            case RGBA: return GL3bc.GL_RGBA;
            case sRGB: return GL3bc.GL_SRGB;
            case sRGBA: return GL3bc.GL_SRGB_ALPHA;
        }
        
        throw(new NullPointerException("The texture format is undefined"));
    }
    
    protected final int getFormat() {
        if (format != null) switch(format) {
            case RGB: return GL3bc.GL_RGB;
            case RGBA: return GL3bc.GL_RGBA;
            case sRGB: return GL3bc.GL_RGB;
            case sRGBA: return GL3bc.GL_RGBA;
        }
        
        throw(new NullPointerException("The texture format is undefined"));
    }

    @Override
    protected void onLoad(final GL3bc gl) {
        gl.glGenTextures(1, tbo);
    }

    @Override
    protected void onUnload(final GL3bc gl) {
        gl.glDeleteTextures(1, tbo);
    }

}
