package fr.amap.lidar.amapvox.visualization.opengl.resources.textures;

import com.jogamp.opengl.GL3bc;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;
import java.io.IOException;
import java.nio.Buffer;

public class Texture2D extends Texture {
    
    private final Buffer image;

    public Texture2D(final String identifier, final int width, final int height,
            final TextureFormat format, final boolean generateMipmaps) {
        super(identifier, width, height, format, generateMipmaps);
        this.image = null;
    }
    
    public Texture2D(final String identifier, final int width, final int height,
            final TextureFormat format, final boolean generateMipmaps, final String path) throws IOException {
        super(identifier, width, height, format, generateMipmaps);
        
        final TextureData data = TextureIO.newTextureData(
                GLProfile.getGL2GL3(), 
                getClass().getResource(path), 
                getInternalFormat(), getFormat(),
                false, path.split("\\.")[1]);
        image = data.getBuffer();
    }
    
    @Override
    protected void onLoad(final GL3bc gl) {
        super.onLoad(gl);
                        
        gl.glBindTexture(GL3bc.GL_TEXTURE_2D, tbo.get(0));
        
        gl.glTexParameteri(GL3bc.GL_TEXTURE_2D, GL3bc.GL_TEXTURE_WRAP_S, GL3bc.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_2D, GL3bc.GL_TEXTURE_WRAP_T, GL3bc.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_2D, GL3bc.GL_TEXTURE_MAG_FILTER, GL3bc.GL_LINEAR);
        
        if (generateMipmaps) {
            gl.glTexParameteri(GL3bc.GL_TEXTURE_2D, GL3bc.GL_TEXTURE_MIN_FILTER, GL3bc.GL_LINEAR_MIPMAP_LINEAR);
        } else {
            gl.glTexParameteri(GL3bc.GL_TEXTURE_2D, GL3bc.GL_TEXTURE_MIN_FILTER, GL3bc.GL_LINEAR);
        }
        
        gl.glTexImage2D(GL3bc.GL_TEXTURE_2D, 0, getInternalFormat(), width, height, 
                0, getFormat(), GL3bc.GL_UNSIGNED_BYTE, image);
        
        if (generateMipmaps) {
            gl.glGenerateMipmap(GL3bc.GL_TEXTURE_2D);
        }
    }
    
    @Override
    protected void onBind(final GL3bc gl) {
        gl.glBindTexture(GL3bc.GL_TEXTURE_2D, tbo.get(0));
    }

    @Override
    protected void onUnbind(final GL3bc gl) {
        gl.glBindTexture(GL3bc.GL_TEXTURE_2D, 0);
    }
    
}
