package fr.amap.lidar.amapvox.visualization.opengl.resources.textures;

import com.jogamp.opengl.GL3bc;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;
import java.io.IOException;
import java.nio.Buffer;
import java.nio.ByteBuffer;

public class TextureCubemap extends Texture {
    
    private final Buffer[] images;
    
    public TextureCubemap(final String identifier, final int dimension,
            final TextureFormat format,final boolean generateMipmaps) throws IOException {
        super(identifier, dimension, dimension, format, generateMipmaps);
        this.images = new ByteBuffer[6];
    }
    
    public TextureCubemap(final String identifier, final int dimension,
            final TextureFormat format, final boolean generateMipmaps,
            final String rightPath, final String leftPath,
            final String topPath, final String bottomPath,
            final String backPath, final String frontPath) throws IOException {
        super(identifier, dimension, dimension, format, generateMipmaps);
        this.images = new ByteBuffer[6];
                
        final TextureData right = TextureIO.newTextureData(
                GLProfile.getGL2GL3(), 
                getClass().getResource(rightPath), 
                getInternalFormat(), getFormat(),
                generateMipmaps, rightPath.split("\\.")[1]);
        images[0] = right.getBuffer();
                
        final TextureData left = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(leftPath), 
                getInternalFormat(), getFormat(),
                generateMipmaps, leftPath.split("\\.")[1]);
        images[1] = left.getBuffer();
        
        final TextureData bottom = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(bottomPath), 
                getInternalFormat(), getFormat(),
                generateMipmaps, bottomPath.split("\\.")[1]);
        images[2] = bottom.getBuffer();
        
        final TextureData top = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(topPath), 
                getInternalFormat(), getFormat(),
                generateMipmaps, topPath.split("\\.")[1]);
        images[3] = top.getBuffer();
        
        final TextureData back = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(backPath), 
                getInternalFormat(), getFormat(),
                generateMipmaps, backPath.split("\\.")[1]);
        images[4] = back.getBuffer();
        
        final TextureData front = TextureIO.newTextureData(
                GLProfile.getGL2GL3(),
                getClass().getResource(frontPath), 
                getInternalFormat(), getFormat(),
                generateMipmaps, frontPath.split("\\.")[1]);
        images[5] = front.getBuffer();
    }
    
    @Override
    protected void onLoad(final GL3bc gl) {
        super.onLoad(gl);
        
        gl.glBindTexture(GL3bc.GL_TEXTURE_CUBE_MAP, tbo.get(0));
        
        final int internalFormat = getInternalFormat();
        final int format = getFormat();
        
        for(int i = 0; i < images.length; i++)
        {
            gl.glTexImage2D(GL3bc.GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 
                0, internalFormat, width, height, 0, format, 
                GL3bc.GL_UNSIGNED_BYTE, images[i]
            );
        }
        
        if (generateMipmaps) {
            gl.glGenerateMipmap(GL3bc.GL_TEXTURE_CUBE_MAP);
            gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_MIN_FILTER, GL3bc.GL_LINEAR_MIPMAP_LINEAR);
        } else {
            gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_MIN_FILTER, GL3bc.GL_LINEAR);
        }
        
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_MAG_FILTER, GL3bc.GL_LINEAR);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_WRAP_S, GL3bc.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_WRAP_T, GL3bc.GL_CLAMP_TO_EDGE);
        gl.glTexParameteri(GL3bc.GL_TEXTURE_CUBE_MAP, GL3bc.GL_TEXTURE_WRAP_R, GL3bc.GL_CLAMP_TO_EDGE); 
    }
    
    @Override
    protected void onBind(final GL3bc gl) {
        gl.glBindTexture(GL3bc.GL_TEXTURE_CUBE_MAP, tbo.get(0));
    }

    @Override
    protected void onUnbind(final GL3bc gl) {
        gl.glBindTexture(GL3bc.GL_TEXTURE_CUBE_MAP, 0);
    }
    
}
