/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.opengl.windows;

import fr.amap.lidar.amapvox.visualization.opengl.util.InputStatus;
import com.jogamp.newt.event.InputEvent;
import com.jogamp.opengl.GL3bc;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLDebugMessage;
import com.jogamp.opengl.GLEventListener;
import fr.amap.lidar.amapvox.visualization.opengl.entities.base.Scene;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class VisualizationGLEventListener implements GLEventListener {

    private Scene scene;
    private final long startTime;
    private long lastTime;
    private boolean newScene;
    private int width, height;
    private GL3bc gl;

    public VisualizationGLEventListener() {
        this.scene = null;
        this.startTime = System.currentTimeMillis();
        this.newScene = false;
        this.gl = null;
    }
    
    public final void setScene(final Scene scene) {
        if (scene != null && gl != null) {
            scene.destroy(gl);
        }
        
        this.scene = scene;
        this.newScene = true;
    }

    public final Scene getScene() {
        return scene;
    }
    
    @Override
    public final void init(final GLAutoDrawable drawable) {
        this.gl = drawable.getGL().getGL3bc();
        
        /* Initialize the visualization context */
        System.setProperty("jogl.disable.openglcore", "true");
        
        System.out.println("Chosen GLCapabilities: " + drawable.getChosenGLCapabilities());
        System.out.println("INIT GL IS: " + gl.getClass().getName());
        System.out.println("GL_VENDOR: " + gl.glGetString(GL3bc.GL_VENDOR));
        System.out.println("GL_RENDERER: " + gl.glGetString(GL3bc.GL_RENDERER));
        System.out.println("GL_VERSION: " + gl.glGetString(GL3bc.GL_VERSION));
                
        drawable.getContext().addGLDebugListener((GLDebugMessage event) -> {
            if (event.getDbgSeverity() == GL3bc.GL_DEBUG_SEVERITY_HIGH) {
                System.out.println("GL Debug: " + event.getDbgMsg());
            }
        });
        
        if (scene != null) {
            scene.init(gl);
        }
    }

    @Override
    public final void display(final GLAutoDrawable drawable) {
        this.gl = drawable.getGL().getGL3bc();
        gl.glClearColor(0.92f, 0.93f, 0.95f, 1.0f);
        
        if (newScene) {
            scene.init(gl);
            scene.rescale(gl, width, height);
            newScene = false;
        }
        
        if (scene != null) {
            long time = startTime - System.currentTimeMillis();
            scene.update(gl, (lastTime - time) * 0.01f);
            lastTime = time;

            scene.display(gl);
        } else {
            gl.glClear(GL3bc.GL_COLOR_BUFFER_BIT);
        }
    }
    
    public final void inputting(final InputEvent event, final InputStatus status) {
        if (scene != null) {
            scene.onInputting(event, status);
        }
    }

    @Override
    public final void reshape(final GLAutoDrawable drawable, final int x, 
            final int y, final int width, final int height) {
        this.gl = drawable.getGL().getGL3bc();
        this.width = width;
        this.height = height;
        
        if (scene != null) {
            scene.rescale(gl, width, height);
        }
    }
    
    @Override
    public final void dispose(final GLAutoDrawable drawable) {
        gl = drawable.getGL().getGL3bc();
        
        if (scene != null) {
            scene.destroy(gl);
        }
    }

}
