/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.visualization.views.voxelfilter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JSlider;
import javax.swing.border.LineBorder;
import org.jdesktop.swingx.JXButton;
import org.jdesktop.swingx.JXComboBox;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXPanel;
import org.jdesktop.swingx.JXTextField;
import fr.amap.lidar.amapvox.visualization.opengl.entities.util.VoxelFilter;
import fr.amap.lidar.amapvox.visualization.opengl.entities.util.VoxelFilteringType;
import fr.amap.lidar.amapvox.visualization.opengl.entities.util.VoxelSpaceFile;
import fr.amap.lidar.amapvox.visualization.util.View;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class JXVoxelFilter extends View {
    
    private List<JXVoxelFilterListener> listeners;
    
    private VoxelSpaceFile voxelFile;
    
    private final int index;
    
    private JXPanel contentPanel;
    private JXPanel bodyPanel;
    private JXPanel closePanel;
    
    private JXPanel propertyPanel;
    private JXLabel propertyLabel;
    private JXComboBox propertyComboBox;
    
    private JXPanel filteringPanel;
    private JXLabel filteringLabel;
    private JXComboBox filteringComboBox;
    private JCheckBox filteringNaNCheckBox;
    
    private JXPanel valuePanel;
    private JXLabel valueLabel;
    private JSlider valueSlider;
    private JXTextField valueTextField;
    
    private JXButton closeButton;
    
    public JXVoxelFilter() throws Exception {
        super();
        this.listeners = new ArrayList();
        final Random random = new Random();
        this.index = random.nextInt();
    }
    
    public final void addListener(final JXVoxelFilterListener listener) {
        listeners.add(listener);
    }
    
    public final void updatePropertyNames(final String[] names) {
        final ComboBoxModel model = new DefaultComboBoxModel(names);
        propertyComboBox.setModel(model);
        propertyComboBox.setSelectedIndex(0);
    }
    
    public final void setVoxelFile(final VoxelSpaceFile voxelFile) {
        this.voxelFile = voxelFile;
    }
    
    public final JXPanel getContentPanel() {
        return contentPanel;
    }
    
    @Override
    protected void initContainer() throws Exception {
        getContentPane().setLayout(new BorderLayout());
    }
    
    @Override
    protected void initComponents() throws Exception {
        /* Init content panel */
        contentPanel = new JXPanel(new BorderLayout());
        contentPanel.setBorder(new LineBorder(Color.BLACK));
        contentPanel.setMaximumSize(new Dimension(10000, 100));
        getContentPane().add(contentPanel, BorderLayout.CENTER);
        
        bodyPanel = new JXPanel(new GridLayout(0, 1));
        contentPanel.add(bodyPanel, BorderLayout.CENTER);
        
        closePanel = new JXPanel(new FlowLayout());
        closeButton = new JXButton(loadIcon("/icons/close.png"));
        closeButton.addActionListener((e) -> {
            for (JXVoxelFilterListener listener : listeners) {
                listener.onClose(getFilter());
            }
        });
        closePanel.add(closeButton);
        contentPanel.add(closePanel, BorderLayout.EAST);

        /* Init property panel */
        propertyPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        bodyPanel.add(propertyPanel);

        /* Init property label */
        propertyLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            propertyLabel.setText(bundle.getString("voxelFilter_propertyLabel") + " :");
        });
        propertyPanel.add(propertyLabel);

        /* Init property combobox */
        propertyComboBox = new JXComboBox();
        propertyComboBox.addItemListener((e) -> {
            updateFilter();
        });
        propertyPanel.add(propertyComboBox);

        /* Init filtering panel */
        filteringPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        bodyPanel.add(filteringPanel);

        /*Init filtering label */
        filteringLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            filteringLabel.setText(bundle.getString("voxelFilter_filteringLabel") + " :");
        });
        filteringPanel.add(filteringLabel);

        /* Init filtering combobox */
        filteringComboBox = new JXComboBox();
        filteringComboBox.addItemListener((e) -> {
            updateFilter();
        });
        addLocaleCallback((locale) -> {
            final ComboBoxModel model = new DefaultComboBoxModel(new String[]{
                bundle.getString("voxelFilter_filteringComboBox_lesser"),
                bundle.getString("voxelFilter_filteringComboBox_equal"),
                bundle.getString("voxelFilter_filteringComboBox_greater"),
                bundle.getString("voxelFilter_filteringComboBox_notequal"),});
            filteringComboBox.setModel(model);
            filteringComboBox.setSelectedIndex(1);
        });
        filteringPanel.add(filteringComboBox);
        
        filteringNaNCheckBox = new JCheckBox("NaN");
        filteringNaNCheckBox.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        filteringNaNCheckBox.addActionListener((e) -> {
            updateFilter();
        });
        filteringPanel.add(filteringNaNCheckBox);

        /* Init value panel */
        valuePanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        bodyPanel.add(valuePanel);

        /* Init value label */
        valueLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            valueLabel.setText(bundle.getString("voxelFilter_valueLabel") + " :");
        });
        valuePanel.add(valueLabel);

        /* Init value slider */
        valueSlider = new JSlider(0, 100, 0);
        valueSlider.addChangeListener((e) -> {
            if (voxelFile != null) {
                final float min = voxelFile.getMinPropertyValue((String) propertyComboBox.getSelectedItem());
                final float max = voxelFile.getMaxPropertyValue((String) propertyComboBox.getSelectedItem());
                final float value = (float) (valueSlider.getValue() / (float) valueSlider.getMaximum());
                valueTextField.setText(Float.toString(min + (max - min) * value));
            }
        });
        valueSlider.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent e) {
                updateFilter();
            }
        });
        valuePanel.add(valueSlider);

        /* Init value text field */
        valueTextField = new JXTextField();
        valueTextField.setText("0.0");
        valueTextField.setEditable(false);
        valueTextField.setPreferredSize(new Dimension(100, valueTextField.getPreferredSize().height));
        valuePanel.add(valueTextField);
    }
    
    private void updateFilter() {
        for (JXVoxelFilterListener listener : listeners) {
            listener.onChange(getFilter());
        }
    }
    
    public final VoxelFilter getFilter() {
        return new VoxelFilter(
                index,
                (String) propertyComboBox.getSelectedItem(),
                (float) (valueSlider.getValue() / (float) valueSlider.getMaximum()),
                VoxelFilteringType.values()[filteringComboBox.getSelectedIndex()],
                filteringNaNCheckBox.isSelected()
        );
    }
    
}
